using System;
using System.Collections.Generic;
using System.Linq;
using AvaloniaApplication4.AdditionalModels;
using AvaloniaApplication4.Models;
using AvaloniaApplication4.Views.ActionPages;
using Microsoft.EntityFrameworkCore;
using ReactiveUI;

namespace AvaloniaApplication4.ViewModels
{
	public class AllProgramsViewModel : ViewModelBase
    {
        GlobalsData globalsData;
        User? user;
        List<RunningProgram> programs = new List<RunningProgram>();
        List<RunningProgram> programsRun = new List<RunningProgram>();
        List<RunningProgram> programsNotRun = new List<RunningProgram>();
        List<RunningProgram> programsRunStatic = new List<RunningProgram>();
        List<RunningProgram> programsNotRunStatic = new List<RunningProgram>();
        string searchProgramsRun = string.Empty;
        string searchProgramsNotRun = string.Empty;
        bool isVisibleFirst = true;
        bool isVisibleSecond = true;
        public AllProgramsViewModel(GlobalsData globalsData, User? user)
        {
            this.globalsData = globalsData;
            this.user = user;
            programs = dataBase.RunningPrograms
                .Include(x => x.IdProgramTemplateNavigation)
                .Include(x => x.IdSecretaryNavigation)
                .Include(x => x.IdStatusNavigation)
                .Include(x => x.IdTutorNavigation)
                .ToList();
            programsRun = programs.Where(x => x.IdStatusNavigation.NameStatus == " ").ToList();
            programsNotRun = programs.Where(x => x.IdStatusNavigation.NameStatus == "" || x.IdStatusNavigation.NameStatus == "").ToList();
            programsNotRunStatic = programsNotRun;
            programsRunStatic = programsRun;
            if (programsRun.Count > 0 ) isVisibleFirst = true;
            else isVisibleFirst= false;
            if (programsNotRun.Count > 0) isVisibleSecond = true;
            else isVisibleSecond = false;
        }
        public void EditAndAddProgram(int id)
        {
            var program = dataBase.RunningPrograms.Include(x => x.IdProgramTemplateNavigation)
                .Include(x => x.IdSecretaryNavigation)
                .Include(x => x.IdStatusNavigation)
                .Include(x => x.IdTutorNavigation)
                .FirstOrDefault(x => x.IdRunningPrograms == id);
            if (program != null)
            {
                globalsData.GlobalUserControl = new EditAndAddProgram(user, ref globalsData, program);
            }
        }
        public List<RunningProgram> ProgramsRun
        {
            get { return programsRun; }
            set
            {
                if (programsRun == value)
                    return;

                programsRun = value;
                OnPropertyChanged("ProgramsRun");
            }
        }
        public List<RunningProgram> ProgramsNotRun
        {
            get { return programsNotRun; }
            set
            {
                if (programsNotRun == value)
                    return;

                programsNotRun = value;
                OnPropertyChanged("ProgramsNotRun");
            }
        }

        public bool IsVisibleFirst { get => isVisibleFirst; set => isVisibleFirst = value; }
        public bool IsVisibleSecond { get => isVisibleSecond; set => isVisibleSecond = value; }
       
        public void AddProgram()
        {
            globalsData.GlobalUserControl = new EditAndAddProgram(user, ref globalsData);
        }
        public string SearchProgramsRun
        {
            get
            {
                return searchProgramsRun;
            }
            set
            {
                searchProgramsRun = value;
                if (searchProgramsRun != null && searchProgramsRun != string.Empty && searchProgramsRun != "")
                {
                    ProgramsRun = programsRunStatic.Where(x=>x.IdProgramTemplateNavigation.NameProgram.ToLower().StartsWith(searchProgramsRun.ToLower()) || x.IdProgramTemplateNavigation.CodeProgram.StartsWith(searchProgramsRun)).ToList();
                }
                else ProgramsRun = programsRunStatic;
            }
        }
        public string SearchProgramsNotRun
        {
            get
            {
                return searchProgramsNotRun;
            }
            set
            {
                searchProgramsNotRun = value;
                if (searchProgramsNotRun != null && searchProgramsNotRun != string.Empty && searchProgramsNotRun != "")
                {
                    ProgramsNotRun = programsNotRunStatic.Where(x => x.IdProgramTemplateNavigation.NameProgram.ToLower().StartsWith(searchProgramsNotRun.ToLower()) || x.IdProgramTemplateNavigation.CodeProgram.StartsWith(searchProgramsNotRun)).ToList();
                }
                else ProgramsNotRun = programsNotRunStatic;
            }
        }
    }
}