using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using AvaloniaApplication4.AdditionalModels;
using AvaloniaApplication4.Models;
using AvaloniaApplication4.Views.ActionPages;
using Microsoft.EntityFrameworkCore;
using ReactiveUI;

namespace AvaloniaApplication4.ViewModels
{
	public class EditAndAddProgramViewModel : ViewModelBase
	{
        GlobalsData globalsData;
        User? user;
        RunningProgram runningProgram;

        List<BindingModules> bindingsModules = new List<BindingModules>();
        List<BindingSubjects> bindingSubjects = new List<BindingSubjects>();

        List<ProgramTemplate> programTemplates;
        int idRunningProgramTemplate;

        bool isRunning = false;
        bool isNew = false;
        string btnContent = " ";

        public int IdRunningProgramTemplate
        {
            get { return idRunningProgramTemplate; }
            set
            {
                if (idRunningProgramTemplate == value)
                    return;
                idRunningProgramTemplate = value;
                OnPropertyChanged("IdRunningProgramTemplate");
            }
        }
        public List<ProgramTemplate> ProgramTemplates { get => programTemplates; set => programTemplates = value; }
        public EditAndAddProgramViewModel(GlobalsData globalsData, User? user)
        {
            this.globalsData = globalsData;
            this.user = user;
            idRunningProgramTemplate = 0;
            runningProgram = new RunningProgram();
            programTemplates = dataBase.ProgramTemplates.ToList();
            isNew = true;

        }

        public EditAndAddProgramViewModel(GlobalsData globalsData, User? user, RunningProgram runningProgram) : this(globalsData, user)
        {
            this.runningProgram = runningProgram;
            programTemplates = dataBase.ProgramTemplates.ToList();
            idRunningProgramTemplate = runningProgram.IdProgramTemplate - 1;
            if (this.runningProgram.IdStatusNavigation.NameStatus == " ")
            {
                isRunning = true;
            }
            else isNew = true;
            var allModules = dataBase.ProgramsWithModules
                .Include(x => x.IdModuleNavigation)
                .Include(x => x.IdRunningProgramNavigation)
                .Include(x => x.IdRunningProgramNavigation.IdProgramTemplateNavigation)
                .Where(x => x.IdRunningProgram == this.runningProgram.IdRunningPrograms)
                .ToList();
            foreach (var item in allModules)
            {
                var currentModule = dataBase.Modules.FirstOrDefault(x => x.IdModule == item.IdModule);
                if (currentModule != null)
                {
                    bindingsModules.Add(new BindingModules(currentModule));
                }
            }
            var allSubjects = dataBase.ProgramsWithSubjects
                .Include(x=>x.IdRunningProgramNavigation)
                .Include(x=>x.IdRunningProgramNavigation.IdProgramTemplateNavigation)
                .Include(x=>x.IdSubjectNavigation)
                .Include(x=>x.IdSubjectNavigation.IdCodeAcademicSubjectNavigation)
                .Where(x=>x.IdRunningProgram == this.runningProgram.IdRunningPrograms && x.IdSubjectNavigation.IdCodeAcademicSubjectNavigation.IdCodeAcademicSubjects == "")
                .ToList();
            foreach (var item in allSubjects)
            {
                var currentSubject = dataBase.Subjects.FirstOrDefault(x=>x.IdSubject == item.IdSubject);
                if (currentSubject != null)
                {
                    bindingSubjects.Add(new BindingSubjects(currentSubject));
                }
            }

        }
        public void AddNewModule()
        {
            var temp = BindingsModules;
            temp.Add(new BindingModules());
            BindingsModules = temp;
        }
        public void AddNewSubject()
        {
            var temp = BindingSubjects1;
            temp.Add(new BindingSubjects());
            BindingSubjects1 = temp;
        }
        public void DeleteOneModule(int idModule)
        {
            var temp = BindingsModules;
            var currentModule = temp.FirstOrDefault(x => x.Id == idModule);
            if (currentModule != null)
            {
                temp.Remove(currentModule);
            }
            BindingsModules = temp;
        }
        public void DeleteOneSubject(int idSubject)
        {
            var temp = BindingSubjects1;
            var currentSubject = temp.FirstOrDefault(x => x.Id == idSubject);
            if (currentSubject != null)
            {
                temp.Remove(currentSubject);
            }
            BindingSubjects1 = temp;
        }
        public void GoBack()
        {
            globalsData.GlobalUserControl = new AllPrograms(user, ref globalsData);
        }
        public void Save()
        {

        }
        public void Delete()
        {

        }
        public void StartProgram()
        {

        }
        
        public string BtnContent { get => btnContent; set => btnContent = value; }
        public List<BindingModules> BindingsModules
        {
            get { return bindingsModules; }
            set
            {
                bindingsModules = value;
                OnPropertyChanged("BindingsModules");
            }
        }
        public List<BindingSubjects> BindingSubjects1
        {
            get { return bindingSubjects; }
            set
            {
                bindingSubjects = value;
                OnPropertyChanged("BindingSubjects1");
            }
        }
        public bool IsRunning { get => isRunning; set => isRunning = value; }
        public bool IsNew { get => isNew; set => isNew = value; }
        public RunningProgram RunningProgram { get => runningProgram; set => runningProgram = value; }

        public class BindingModules: ViewModelBase
        {
            List<Models.Module> allModules;
            Models.Module module;
            int id;
            public BindingModules()
            {
                allModules = dataBase.Modules.ToList();
                module = allModules.First();
                id = module.IdModule;
            }

            public BindingModules(Models.Module module): this()
            {
                this.module = dataBase.Modules.FirstOrDefault(x=>x.IdModule == module.IdModule);
                id = this.module.IdModule;
            }

            public List<Models.Module> AllModules { get => allModules; set => allModules = value; }
            public Models.Module Module { get => module; set => module = value; }
            public int Id { get => id; set => id = value; }
        }
        public class BindingSubjects: ViewModelBase
        {
            List<Subject> allSubjects;
            Subject subject;
            int id;

            public BindingSubjects()
            {
                allSubjects = dataBase.Subjects.Include(x=>x.IdCodeAcademicSubjectNavigation).Where(x=>x.IdCodeAcademicSubjectNavigation.IdCodeAcademicSubjects == "").ToList();
                subject = allSubjects.First();
                id = subject.IdSubject;
            }

            public BindingSubjects(Subject subject): this() 
            {
                this.subject = dataBase.Subjects.FirstOrDefault(x=>x.IdSubject == subject.IdSubject);
                id = this.subject.IdSubject;
            }

            public List<Subject> AllSubjects { get => allSubjects; set => allSubjects = value; }
            public Subject Subject { get => subject; set => subject = value; }
            public int Id { get => id; set => id = value; }
        }
    }
}