using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Reactive.Linq;
using AvaloniaApplication4.AdditionalModels;
using AvaloniaApplication4.Models;
using AvaloniaApplication4.Views.ActionPages;
using Microsoft.EntityFrameworkCore;
using ReactiveUI;
using Tmds.DBus.Protocol;

namespace AvaloniaApplication4.ViewModels
{
    public class EditAndAddStudentViewModel : ViewModelBase
    {
        Student student;
        private bool isNew = false;
        List<StudentsEducation> allEducations = new List<StudentsEducation>();
        List<RunningProgram> allPrograms = new List<RunningProgram>();
        private int idEducationStudent = 1;
        private int idProgramStudent = 1;
        GlobalsData globalsData;
        User? user;
        public EditAndAddStudentViewModel(User? user, ref GlobalsData globalsData)
        {
            student = new Student() { IdStudent = 0, Name = "", Surname = "", Patronumic = "", Birthday = DateOnly.FromDateTime(DateTime.Now), IdEducation = 1 };
            isNew = true;
            allEducations = dataBase.StudentsEducations.ToList();
            allPrograms = dataBase.RunningPrograms
                .Include(x=>x.IdProgramTemplateNavigation)
                .ToList();
            idEducationStudent = student.IdEducation - 1;
            this.globalsData = globalsData;
            this.user = user;
        }
        public EditAndAddStudentViewModel(User? user, Student student, ref GlobalsData globalsData)
        {
            this.student = student;
            isNew = false;
            allEducations = dataBase.StudentsEducations.ToList();
            idEducationStudent = student.IdEducation - 1;
            this.globalsData = globalsData;
            this.user = user;
            allPrograms = dataBase.RunningPrograms
                .Include(x => x.IdProgramTemplateNavigation)
                .ToList();
            var areEquals = dataBase.ProgramsWithStudents
                    .Include(x => x.IdRunningProgramNavigation)
                    .Include(x => x.IdRunningProgramNavigation.IdProgramTemplateNavigation)
                    .Include(x => x.IdStudentNavigation)
                    .FirstOrDefault(x => x.IdStudent == student.IdStudent);
            if (areEquals != null)
            {
                idProgramStudent = areEquals.IdRunningProgram - 1;
            }
        }

        public bool IsNew { get => isNew; set => isNew = value; }
        public List<StudentsEducation> AllEducations
        {
            get { return allEducations; }
            set
            {
                if (allEducations == value)
                    return;

                allEducations = value;
                OnPropertyChanged("AllEducations");
            }
        }
        public int IdEducationStudent
        {
            get { return idEducationStudent; }
            set
            {
                if (idEducationStudent == value)
                    return;

                idEducationStudent = value;
                Student.IdEducation = value;
                OnPropertyChanged("IdEducationStudent");
            }
        }
        public Student Student
        {
            get { return student; }
            set
            {
                if (student == value)
                    return;

                student = value;
                OnPropertyChanged("Student");
            }
        }

        public List<RunningProgram> AllPrograms { get => allPrograms; set => allPrograms = value; }
        public int IdProgramStudent
        {
            get { return idProgramStudent; }
            set
            {
                if (idProgramStudent == value)
                    return;
                idProgramStudent = value;
                OnPropertyChanged("IdProgramStudent");
            }
        }

        public void Delete()
        {
            if (!isNew)
            {
                var obj = dataBase.Students.FirstOrDefault(x => x.IdStudent == Student.IdStudent);
                if (obj != null)
                {
                    var areEquals = dataBase.ProgramsWithStudents
                    .Include(x => x.IdRunningProgramNavigation)
                    .Include(x => x.IdStudentNavigation)
                    .Include(x => x.IdRunningProgramNavigation.IdProgramTemplateNavigation)
                    .FirstOrDefault(x => x.IdStudent == student.IdStudent);
                    if (areEquals != null)
                    {
                        dataBase.ProgramsWithStudents.Remove(areEquals);
                    }
                    dataBase.Students.Remove(obj);
                    dataBase.SaveChanges();
                    GoBack();
                }
            }
        }
        public void Save()
        {
            // 
            if (isNew)
            {
                dataBase.Students.Add(Student);
                dataBase.SaveChanges();
                ProgramsWithStudent programsWithStudent = new ProgramsWithStudent() { IdRunningProgram = IdProgramStudent + 1, IdStudent = Student.IdStudent};
                dataBase.ProgramsWithStudents.Add(programsWithStudent);
                dataBase.SaveChanges();
                WriteToLog(user.IdUser, "()   " + Student.Surname + " " + Student.Name + " " + Student.Patronumic);
            }
            else
            {
                var obj = dataBase.Students.FirstOrDefault(x=>x.IdStudent == Student.IdStudent);
                if (obj != null)
                {
                    obj.Name = Student.Name;
                    obj.Surname = Student.Surname;
                    obj.Patronumic = Student.Patronumic;
                    obj.Birthday = Student.Birthday;
                    obj.IdEducation = IdEducationStudent + 1;
                    dataBase.SaveChanges();
                    WriteToLog(user.IdUser, "()    " + obj.Surname + " " + obj.Name + " " + obj.Patronumic);
                }
                var areEquals = dataBase.ProgramsWithStudents
                    .Include(x => x.IdRunningProgramNavigation)
                    .Include(x => x.IdStudentNavigation)
                    .Include(x => x.IdRunningProgramNavigation.IdProgramTemplateNavigation)
                    .FirstOrDefault(x => x.IdStudent == student.IdStudent);
                if (areEquals != null)
                {
                    areEquals.IdRunningProgram = IdProgramStudent + 1;
                    dataBase.SaveChanges();
                    WriteToLog(user.IdUser, "()    " + obj.Surname + " " + obj.Name + " " + obj.Patronumic);
                }
            }
        }
        public void GoBack()
        {
            globalsData.GlobalUserControl = new AllStudents(user, ref globalsData);
        }
    }
}