using System;
using System.Collections.Generic;
using System.Linq;
using Avalonia;
using Avalonia.Controls;
using AvaloniaApplication4.AdditionalModels;
using AvaloniaApplication4.Models;
using AvaloniaApplication4.Views;
using AvaloniaApplication4.Views.ActionPages;
using Microsoft.EntityFrameworkCore;
using ReactiveUI;

namespace AvaloniaApplication4.ViewModels
{
	public class MainPageViewModel : ViewModelBase
	{
		private User? user;
        private UserControl menuUserControl = new MenuPage();
        private List<MenuElement> basicActionsMenuElements = new List<MenuElement>();
        private List<MenuElement> additionalActionsMenuElements = new List<MenuElement>();
        private bool isVisibleAdditional = true;
        private bool isVisibleBase = true;
        private UserControl activePage;

        GlobalsData globalsData = new();
        public GlobalsData GlobalsData { get => globalsData; set => globalsData = value; }

        public UserControl ActivePage
        {
            get { return activePage; }
            set
            {
                if (activePage == value)
                    return;

                activePage = value;
                OnPropertyChanged("ActivePage");
            }
        }
        public MainPageViewModel(User? user)
        {
            this.user = user;
            SetActionsOnMenu();
            globalsData = new GlobalsData();
            GlobalsData.GlobalUserControl = new StartPage(user);
        }
        public MainPageViewModel()
        {
            
        }
        public void SetActivePage(int idAction)
        {
            switch (idAction)
            {
                case 0:
                    GlobalsData.GlobalUserControl = new MySubjects(user, ref globalsData);
                    break;
                case 1:
                    // ,  
                    GlobalsData.GlobalUserControl = new MyPrograms(user, ref globalsData, 0);
                    break;
                case 2:
                    GlobalsData.GlobalUserControl = new AllPrograms(user, ref globalsData);
                    break;
                case 3:
                    GlobalsData.GlobalUserControl = new Logs();
                    break;
                case 4:
                    GlobalsData.GlobalUserControl = new AllStudents(user, ref globalsData);
                    break;
                case 5:
                    GlobalsData.GlobalUserControl = new AllEmployees(user, ref globalsData);
                    break;
                case 6:
                    GlobalsData.GlobalUserControl = new AllSubjectsModulesAndProgramTemplates(user, ref globalsData);
                    break;
                default:
                    GlobalsData.GlobalUserControl = new StartPage(user); 
                    break;                                       
            }
        }
        private void SetActionsOnMenu()
        {
            if (user != null)
            {
                basicActionsMenuElements.Clear();
                additionalActionsMenuElements.Clear();
                if (dataBase.ProgramsWithSubjects.Where(x=>x.IdTeacher == user.IdUser).ToList().Count() > 0)
                {
                    basicActionsMenuElements.Add
                        (
                            new MenuElement() { IdElement = 0, NameElement = " " }
                        );
                }
                if (dataBase.RunningPrograms.Where(x=>x.IdTutor == user.IdUser || x.IdSecretary == user.IdUser).ToList().Count() > 0)
                {
                    basicActionsMenuElements.Add
                        (
                            new MenuElement() { IdElement = 1, NameElement = " " }
                        );
                }
                if (basicActionsMenuElements.Count <= 0) isVisibleBase = false;
                List<UsersWithRole> roles = dataBase.UsersWithRoles.Include(x=>x.IdRoleNavigation).Where(x=>x.IdUser == user.IdUser).ToList();
                if (roles.Count > 0)
                {
                    foreach (var role in roles)
                    {
                        if (role.IdRoleNavigation.RoleName == "")
                        {
                            bool flag = false;
                            foreach (var x in basicActionsMenuElements)
                            {
                                if (x.IdElement == 1) flag = true; break;
                            }
                            if (!flag)
                            {
                                basicActionsMenuElements.Add
                                (
                                    new MenuElement() { IdElement = 1, NameElement = " " }
                                );
                                isVisibleBase = true;
                            }
                            additionalActionsMenuElements = new List<MenuElement>()
                            {
                                new MenuElement() { IdElement = 2, NameElement = " " },
                                new MenuElement() { IdElement = 3, NameElement = " " },
                                new MenuElement() { IdElement = 4, NameElement = " " },
                                new MenuElement() { IdElement = 5, NameElement = " " },
                                new MenuElement() { IdElement = 6, NameElement = " " }
                            };
                        }
                        if (role.IdRoleNavigation.RoleName == "  ")
                        {
                            bool flag = false;
                            foreach (var x in basicActionsMenuElements)
                            {
                                if (x.IdElement == 1) flag = true; break;
                            }
                            if (!flag)
                            {
                                basicActionsMenuElements.Add
                                (
                                    new MenuElement() { IdElement = 1, NameElement = " " }
                                );
                                isVisibleBase = true;
                            }
                            additionalActionsMenuElements = new List<MenuElement>()
                            {
                                new MenuElement() { IdElement = 2, NameElement = " " },
                                new MenuElement() { IdElement = 4, NameElement = " " }
                            };
                        }
                    }
                }
                else IsVisibleAdditional = false;

            }
        }
        public void ToAccount()
        {
            if (user != null)
            {
                GlobalsData.GlobalUserControl = new AccountPage(user);
            }
        }
        public void LogOut()
        {
            //Application.Current.ApplicationLifetime.
        }
        public UserControl MenuUserControl
        {
            get { return menuUserControl; }
            set
            {
                if (menuUserControl == value)
                    return;

                menuUserControl = value;
                OnPropertyChanged("MenuUserControl");
            }
        }
        public List<MenuElement> BasicActionsMenuElements
        {
            get { return basicActionsMenuElements; }
            set
            {
                if (basicActionsMenuElements == value)
                    return;

                basicActionsMenuElements = value;
                OnPropertyChanged("BasicActionsMenuElements");
            }
        }
        public List<MenuElement> AdditionalActionsMenuElements
        {
            get { return additionalActionsMenuElements; }
            set
            {
                if (additionalActionsMenuElements == value)
                    return;

                additionalActionsMenuElements = value;
                OnPropertyChanged("AdditionalActionsMenuElements");
            }
        }
        public bool IsVisibleAdditional { get => isVisibleAdditional; set => isVisibleAdditional = value; }
        public bool IsVisibleBase { get => isVisibleBase; set => isVisibleBase = value; }
    }
}