using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Reactive;
using System.Reflection;
using System.Threading.Tasks;
using Avalonia;
using Avalonia.Controls;
using Avalonia.Controls.ApplicationLifetimes;
using Avalonia.Platform.Storage;
using AvaloniaApplication4.AdditionalModels;
using AvaloniaApplication4.Models;
using AvaloniaApplication4.Views;
using AvaloniaApplication4.Views.ActionPages;
using AvaloniaApplication4.Views.StudyLists;
using AvaloniaUI.PrintToPDF;
using Microsoft.EntityFrameworkCore;
using ReactiveUI;
using static AvaloniaApplication4.ViewModels.BindingElement;
using static AvaloniaApplication4.ViewModels.EditAndAddProgramViewModel;

namespace AvaloniaApplication4.ViewModels.StudyListsViewModels
{
    public class DiffCreditViewModel : ViewModelBase
    {
        GlobalsData globalsData;
        User? user;
        Models.ProgramsWithSubject subject;
        List<StudyListSubjectsStudent> allGrade;
        List<BindingElement> allGradesBinding = new List<BindingElement>();
        BindingElement forBindingList;
        int idAction;

        List<User> allUsers;
        User teacher;
        bool isAdmin = false;

        public DiffCreditViewModel(GlobalsData globalsData, User? user, ProgramsWithSubject subject, int idAction)
        {
            this.idAction = idAction;
            this.subject = subject;
            this.globalsData = globalsData;
            this.user = user;

            //  ,        
            List<UsersWithRole> all = dataBase.UsersWithRoles.Include(x=>x.IdRoleNavigation).Where(x=>x.IdUser == user.IdUser).ToList();
            if (all.Count > 0) isAdmin = true;

            allGrade = dataBase.StudyListSubjectsStudents.Include(x=>x.IdStudentNavigation)
                .Include(x=>x.IdSubjectNavigation)
                .Where(x=>x.IdSubject == subject.IdSubject)
                .ToList();
            var allStudents = dataBase.ProgramsWithStudents.Include(x => x.IdStudentNavigation).Where(x => x.IdRunningProgram == subject.IdRunningProgram).ToList();
            allUsers = dataBase.Users.ToList();
            //    ,       
            if (allGrade.Count <= 0)
            {
                foreach (var student in allStudents)
                {
                    allGrade.Add(new StudyListSubjectsStudent { IdStudent = student.IdStudent, IdSubject = subject.IdSubject, Grade = null });
                }
            }
            else
            {
                //    ,   
                foreach(var student in allGrade)
                {
                    var currentStudent = allStudents.FirstOrDefault(x => x.IdStudent == student.IdStudent);
                    allStudents.Remove(currentStudent);
                }
                if (allStudents != null)
                {
                    // 
                    foreach (var student in allStudents)
                    {
                        allGrade.Add(new StudyListSubjectsStudent { IdStudent = student.IdStudent, IdSubject = subject.IdSubject, Grade = null });
                    }
                }
            }
            var allGradeInDatabase = dataBase.StudyListSubjectsStudents.Include(x => x.IdStudentNavigation)
                .Include(x => x.IdSubjectNavigation)
                .Where(x => x.IdSubject == subject.IdSubject)
                .ToList();
            foreach (var student in allGrade)
            {
                if (!allGradeInDatabase.Contains(student)) dataBase.StudyListSubjectsStudents.Add(student);
            }
            dataBase.SaveChanges();
            allGrade = dataBase.StudyListSubjectsStudents.Include(x => x.IdStudentNavigation)
                .Include(x => x.IdSubjectNavigation)
                .Include(x=>x.GradeNavigation)
                .Where(x => x.IdSubject == subject.IdSubject)
                .ToList();
            teacher = dataBase.Users.FirstOrDefault(x => x.IdUser == subject.IdTeacher);
            foreach (var item in allGrade)
            {
                allGradesBinding.Add(new BindingElement(user, item, allGrade, subject,globalsData,idAction));
            }
            forBindingList = allGradesBinding.First();
            
        }
        
        public ProgramsWithSubject Subject { get => subject; set => subject = value; }
        public List<StudyListSubjectsStudent> AllGrade
        {
            get { return allGrade; }
            set
            {
                if (allGrade == value)
                    return;
                allGrade = value;
                OnPropertyChanged("AllGrade");
            }
        }
        public List<BindingElement> AllGradesBinding
        {
            get { return allGradesBinding; }
            set
            {
                if (allGradesBinding == value)
                    return;
                allGradesBinding = value;
                OnPropertyChanged("AllGradesBinding");
            }
        }

        public BindingElement ForBindingList { get => forBindingList; set => forBindingList = value; }
        public List<User> AllUsers { get => allUsers; set => allUsers = value; }
        public bool IsAdmin { get => isAdmin; set => isAdmin = value; }
        public void SaveToFile()
        {
            Window window = new PDFPrintDiffCredit(AllGradesBinding, forBindingList.Statistics, Subject);
            SavePDF(window);
        }
        
        public User Teacher
        {
            get { return teacher; }
            set
            {
                if (teacher == value)
                    return;
                teacher = value;
                var currentSubj = dataBase.ProgramsWithSubjects.FirstOrDefault(x=>x.IdSubject == Subject.IdSubject);
                if (currentSubj != null)
                {
                    currentSubj.IdTeacher = teacher.IdUser;
                    dataBase.SaveChanges();
                }
                OnPropertyChanged("Teacher");
            }
        }

        public void GoBack()
        {
            if (idAction == 0)
            {
                globalsData.GlobalUserControl = new MySubjects(user, ref globalsData);
            }
            else if (idAction == 1)
            {
                globalsData.GlobalUserControl = new MyPrograms(user, ref globalsData, 0);
            }
        }
    }
}